# Investigating best probe frequency 

# %% Import Declaration
import matplotlib.pyplot as plt
import setuptools
import seaborn as sns
import matplotlib as mpl

import json
import os


import numpy as np
import pandas as pd
import scipy as sp
import skrf as rf
from qutip import *
from numpy import pi

from scipy.optimize import curve_fit
from scipy import special as sp
from pathlib import Path
from lmfit.models import LorentzianModel, gaussian, ConstantModel, LinearModel
from lmfit import fit_report
from scipy.signal import savgol_filter    

import warnings
warnings.filterwarnings("ignore")
# %% Global plt.plot Setting 

# plt.style.use('default')
plt.style.use('seaborn-paper')
current_palette = sns.color_palette(palette=None)
sns.set_palette(palette=None)
# sns.palplot(current_palette)



mpl.rcParams.update({'figure.figsize': (8, 6)})
mpl.rcParams['figure.dpi']= 200

SMALL_SIZE = 10*2
MEDIUM_SIZE = 12*2
BIGGER_SIZE = 14*2

plt.rc('font', size=SMALL_SIZE)          # controls default text sizes
# plt.rcParams["font.weight"] = "bold"
plt.rcParams['axes.linewidth'] = 3         # set axes weight value globally
plt.rc('axes', titlesize=SMALL_SIZE)     # fontsize of the axes title
plt.rc('axes', labelsize=MEDIUM_SIZE)    # fontsize of the x and y labels
plt.rc('xtick', labelsize=SMALL_SIZE)    # fontsize of the tick labels
plt.rc('ytick', labelsize=SMALL_SIZE)    # fontsize of the tick labels
plt.rc('legend', fontsize=SMALL_SIZE)    # legend fontsize
plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title


tick_width = 2
tick_length = 10

plt.rcParams["xtick.major.width"] = tick_width
plt.rcParams["xtick.minor.width"] = tick_width
plt.rcParams["xtick.major.size"] = tick_length
plt.rcParams["xtick.minor.size"] = tick_length/2

plt.rcParams["ytick.major.width"] = tick_width
plt.rcParams["ytick.minor.width"] = tick_width
plt.rcParams["ytick.major.size"] = tick_length
plt.rcParams["ytick.minor.size"] = tick_length/2


# mpl.rcParams.update(mpl.rcParamsDefault)
from scipy.constants import pi, h, hbar, elementary_charge, Boltzmann, speed_of_light

# %% Function def
def find_closest(array, value, axis=None):
    """ Finds the closest elements in array to value along an axis.

    Parameters
    ----------
    array : ndarray, list
        A 1D or 2D array or list.
    value : scalar
        A scalar value to compare.
    axis : None, 0, 1, optional
        Recommended to provide one for 2D array. The default is None.

    Returns
    -------
    out_array : ndarray, one less dimention than array
        Contains the closest values along the axis specified. Not necessarili along the same splice.
    idx : scalar or ndarray
        The indices of the closest elements along axis.

    """
    array = np.asarray(array)
    idx = np.argmin(np.abs(array-value), axis=axis)
    out_array = np.take(array, idx, axis=axis)
    if np.size(idx) > 1:
        out_array = np.diagonal(out_array)
    return out_array, idx


# %% Dataset load

Freq_shift = pd.read_csv("sim_data_ideal\Fshift.txt", delimiter='\t', skiprows=0, float_precision='round_trip').to_numpy()
Pin_linspace = Freq_shift[:,0] #power sweep used in sim

Vin_df  = pd.read_csv("sim_data_ideal\Vin.txt", delimiter=',', skiprows=0)
Vin = Vin_df.to_numpy()
Vin = (Vin.reshape(161, 701, 4)) # Reshaped to [power, freq, real, imag]

Vout_S_df = pd.read_csv("sim_data_ideal\Vout_singlet.txt", delimiter=',', skiprows=0)
Vout_S = Vout_S_df.to_numpy()
test = Vout_S_df.to_numpy()
Vout_S = (Vout_S.reshape(161, 701, 4)) # Reshaped to [power, freq, real, imag]


Vout_T_df = pd.read_csv("sim_data_ideal\Vout_triplet.txt", delimiter=',', skiprows=0)
Vout_T = Vout_T_df.to_numpy()
Vout_T = Vout_T.reshape(161, 701, 4) # Reshaped to [power, freq, real, imag]

# %% BPSK freq.

bare_res_freq = 6.91e9
mid_probe_freq = bare_res_freq - (Freq_shift[:,1]*1e6)/2

idx_probe_mid = []
freq_probe_mid = []
for Probe in mid_probe_freq:
    val,idx = find_closest(Vout_S[0,:,1], Probe)
    idx_probe_mid.append(idx)
    freq_probe_mid.append(val)


# %% Responds for BPSK and BASK

idx_probe_bare = []
freq_probe_bare = bare_res_freq

val_bare,idx_bare = find_closest(Vout_S[0,:,1], freq_probe_bare)


Vout_S_bare = Vout_S[:,idx_bare, 2] + 1j*Vout_S[:,idx_bare, 3]
Vout_T_bare = Vout_T[:,idx_bare, 2] + 1j*Vout_T[:,idx_bare, 3]

Vout_S_mid = []
Vout_T_mid = []
Vin_mid = []

for P_idx, idx_mid in enumerate(idx_probe_mid):
    Vout_S_mid.append(Vout_S[P_idx,idx_mid, 2] + 1j*Vout_S[P_idx,idx_mid, 3])
    Vout_T_mid.append(Vout_T[P_idx,idx_mid, 2] + 1j*Vout_T[P_idx,idx_mid, 3])
    Vin_mid.append(Vin[P_idx,idx_mid, 2] + 1j*Vin[P_idx,idx_mid, 3])
    

#%% Power between BPSK and BASK

A_resp_mid_freq_Vsqr = abs(np.subtract(Vout_S_mid, Vout_T_mid))**2  ## [V^2]
A_resp_bare_freq_Vsqr = abs(np.subtract(Vout_S_bare, Vout_T_bare))**2 ## [V^2]

P_resp_mid_freq = A_resp_mid_freq_Vsqr/50 # [Watts]
P_resp_bare_freq = A_resp_bare_freq_Vsqr/50


plt.figure(figsize=(6,5))
plt.tight_layout() 
plt.semilogy(Pin_linspace, P_resp_bare_freq, color='blue', linewidth=4, label='BASK' )
plt.semilogy(Pin_linspace, P_resp_mid_freq, color='red', linewidth=4, label='BPSK')


plt.ylabel('$P_{RF}|S_{21,|T\\rangle} - S_{21,|S\\rangle}|^2$ [W]', fontsize=20)
plt.xlabel('$P_{RF}$ [dBm]')

plt.legend(fontsize=16, loc=0)
plt.xlim([-140,-65])
plt.ylim([1E-20, 1E-10])

#%% State seperation factor
Vin_linspace = 10**((Pin_linspace-10)/20) 

sep_mid = abs(np.subtract(Vout_S_mid, Vout_T_mid)/Vin_linspace)**2  # [V/V]
sep_bare = abs(np.subtract(Vout_S_bare, Vout_T_bare)/Vin_linspace)**2 # [V/V]

plt.figure(figsize=(6,5))
plt.semilogy(Pin_linspace, sep_bare, color='blue', linewidth=4, label='BASK' )
plt.semilogy(Pin_linspace, sep_mid, color='red', linewidth=4, label='BPSK')

plt.ylabel('$|S_{21,|T\\rangle} - S_{21,|S\\rangle}|^2$')
plt.xlabel('$P_{RF}$ [dBm]')
plt.xlim([-140,-65])
plt.legend(fontsize=16)




