# -*- coding: utf-8 -*-
"""
Created on Sun Feb 26 20:48:06 2023

@author: Bagas Prabowo
"""

# %% Import Packages Declaration

import matplotlib.pyplot as plt
import setuptools
import seaborn as sns
import matplotlib as mpl

import numpy as np
import pandas as pd
import scipy as sp
import skrf as rf
from qutip import *
from numpy import pi

from scipy.optimize import curve_fit
from colour import Color
from matplotlib.colors import LinearSegmentedColormap

from lmfit.models import LorentzianModel, GaussianModel, ConstantModel, LinearModel
from lmfit import fit_report

from matplotlib.ticker import AutoMinorLocator


# %% Global plt.plot Setting 

# plt.style.use('default')
plt.style.use('seaborn-paper')
current_palette = sns.color_palette(palette=None)
sns.set_palette(palette=None)
# sns.palplot(current_palette)
import matplotlib.pylab as pl


mpl.rcParams.update({'figure.figsize': (8, 6 )})
mpl.rcParams['figure.dpi']= 200

scale = 2
SMALL_SIZE = 10*scale
MEDIUM_SIZE = 12*scale
BIGGER_SIZE = 14*scale

plt.rc('font', size=SMALL_SIZE)          # controls default text sizes
# plt.rcParams["font.weight"] = "bold"
plt.rcParams['axes.linewidth'] = 5       # set axes weight value globally
plt.rc('axes', titlesize=SMALL_SIZE)     # fontsize of the axes title
plt.rc('axes', labelsize=MEDIUM_SIZE)    # fontsize of the x and y labels
plt.rc('xtick', labelsize=SMALL_SIZE)    # fontsize of the tick labels
plt.rc('ytick', labelsize=SMALL_SIZE)    # fontsize of the tick labels
plt.rc('legend', fontsize=SMALL_SIZE)    # legend fontsize
plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title

# mpl.rcParams.update(mpl.rcParamsDefault)
tick_width = 4
tick_length = 10
tick_length = 0

plt.rcParams["xtick.major.width"] = tick_width
plt.rcParams["xtick.minor.width"] = tick_width
plt.rcParams["xtick.major.size"] = tick_length
plt.rcParams["xtick.minor.size"] = tick_length/2

plt.rcParams["ytick.major.width"] = tick_width
plt.rcParams["ytick.minor.width"] = tick_width
plt.rcParams["ytick.major.size"] = tick_length
plt.rcParams["ytick.minor.size"] = tick_length/2

# %% Universal Constant and Function Declaration

from scipy.constants import pi, h, hbar, elementary_charge, Boltzmann, speed_of_light

from scipy.special.cython_special import ellipeinc

# %% Plot figure 1

s21_mag_resp1 = np.loadtxt("TQE_Figure_1\s21mag_resp1.txt", delimiter='\t', skiprows=1)
s21_mag_resp2 = np.loadtxt("TQE_Figure_1\s21mag_resp2.txt", delimiter='\t', skiprows=1)

s21_phase_resp1 = np.loadtxt("TQE_Figure_1\s21_phase_resp1.txt", delimiter='\t', skiprows=1)
s21_phase_resp2 = np.loadtxt("TQE_Figure_1\s21_phase_resp2.txt", delimiter='\t', skiprows=1)



plt.figure(figsize=(6,4))
plt.plot(s21_mag_resp1[:,0],s21_mag_resp1[:,1], color='#F17720', linewidth=5)
plt.plot(s21_mag_resp2[:,0],s21_mag_resp2[:,1], color='#0474BA', linewidth=5)

# plt.grid()
plt.ylabel('$S_{21}$ [dB]')
plt.xlabel('Frequency')


plt.figure(figsize=(6,4))
plt.plot(s21_phase_resp1[:,0],s21_phase_resp1[:,1], color='#F17720', linewidth=5)
plt.plot(s21_phase_resp2[:,0],s21_phase_resp2[:,1], color='#0474BA', linewidth=5)

plt.ylabel('Phase [Deg.]')
plt.xlabel('Frequency')



# %% Plot figure 1


# fig, ax1 = plt.subplots(figsize=(5,4))
# ax2 = ax1.twinx()

# ax1.plot(s21_mag_resp1[:,0],s21_mag_resp1[:,1], color='blue', linewidth=5)
# ax1.plot(s21_mag_resp2[:,0],s21_mag_resp2[:,1], color='#ff8843', linewidth=5)

# ax2.plot(s21_phase_resp1[:,0],s21_phase_resp1[:,1],'--', color='blue', linewidth=3)
# ax2.plot(s21_phase_resp2[:,0],s21_phase_resp2[:,1],'--', color='#ff8843', linewidth=3)

# plt.ylabel('Phase [Deg.]')
# plt.xlabel('Frequency')




