# -*- coding: utf-8 -*-
"""
Plot s21 sim behavior with quantum capacitance
"""

# %% Import Packages Declaration

import matplotlib.pyplot as plt
import setuptools
import seaborn as sns
import matplotlib as mpl

import numpy as np
import pandas as pd
import scipy as sp
import skrf as rf
from qutip import *
from numpy import pi

from scipy.optimize import curve_fit
from colour import Color
from matplotlib.colors import LinearSegmentedColormap

from lmfit.models import LorentzianModel, GaussianModel, ConstantModel, LinearModel
from lmfit import fit_report

from matplotlib.ticker import AutoMinorLocator


# %% Global plt.plot Setting 

# plt.style.use('default')
plt.style.use('seaborn-paper')
current_palette = sns.color_palette(palette=None)
sns.set_palette(palette=None)
# sns.palplot(current_palette)
import matplotlib.pylab as pl


mpl.rcParams.update({'figure.figsize': (8, 6 )})
mpl.rcParams['figure.dpi']= 250

SMALL_SIZE = 10*2
MEDIUM_SIZE = 12*2
BIGGER_SIZE = 14*2

plt.rc('font', size=SMALL_SIZE)          # controls default text sizes
# plt.rcParams["font.weight"] = "bold"
plt.rcParams['axes.linewidth'] = 3         # set axes weight value globally
plt.rc('axes', titlesize=SMALL_SIZE)     # fontsize of the axes title
plt.rc('axes', labelsize=MEDIUM_SIZE)    # fontsize of the x and y labels
plt.rc('xtick', labelsize=SMALL_SIZE)    # fontsize of the tick labels
plt.rc('ytick', labelsize=SMALL_SIZE)    # fontsize of the tick labels
plt.rc('legend', fontsize=SMALL_SIZE)    # legend fontsize
plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title

# mpl.rcParams.update(mpl.rcParamsDefault)
tick_width = 2
tick_length = 10

plt.rcParams["xtick.major.width"] = tick_width
plt.rcParams["xtick.minor.width"] = tick_width
plt.rcParams["xtick.major.size"] = tick_length
plt.rcParams["xtick.minor.size"] = tick_length/2

plt.rcParams["ytick.major.width"] = tick_width
plt.rcParams["ytick.minor.width"] = tick_width
plt.rcParams["ytick.major.size"] = tick_length
plt.rcParams["ytick.minor.size"] = tick_length/2

# %% Universal Constant and Function Declaration

from scipy.constants import pi, h, hbar, elementary_charge, Boltzmann, speed_of_light

from scipy.special.cython_special import ellipeinc



# %% Plot for freq. shift
freq_shift = np.loadtxt("Ideal_data\Freq_shift_ideal.txt", delimiter='\t', skiprows=1)

plt.figure(figsize=(6,2.5))
plt.plot(freq_shift[:,0],freq_shift[:,1], color='blue', linewidth=4)

plt.grid()
plt.ylabel('$\Delta$F [MHz]')
plt.xlabel('$P_{RF}$ [dBm]')
plt.xlim([-140,-65])
plt.ylim([0,6])

# %% Plot for S21 Sweep

S21_singlet_mag = np.loadtxt("Ideal_data\s21_singlet_resp_mag.txt", delimiter='\t')
S21_singlet_phase = np.loadtxt("Ideal_data\s21_singlet_resp_phase.txt", delimiter='\t')

S21_triplet_mag = np.loadtxt("Ideal_data\S21_bare_resp_mag.txt", delimiter='\t', skiprows=1)
S21_triplet_phase = np.loadtxt("Ideal_data\S21_bare_resp_phase.txt", delimiter='\t', skiprows=1)

start_index = 0
slice_size = 701

Pin_step = 0.5 # data resolution 
Pin_skip = 5 # How many dbm steps to plot
skip_size = int((Pin_skip/Pin_step)*slice_size)

S21_sliced_mag = []
S21_sliced_phase = []

while start_index < len(S21_singlet_mag):
    S21_sliced_mag.append(S21_singlet_mag[start_index:start_index+slice_size])
    S21_sliced_phase.append(S21_singlet_phase[start_index:start_index+slice_size])


    start_index += slice_size + skip_size


n =  len(S21_sliced_mag)
colors = pl.cm.Blues(np.linspace(0.4,0.75,n))

plt.figure(figsize=(6,2.5))
start_idx=0
while start_idx < n-1:
    plt.plot(S21_sliced_mag[0][:,1]*1E-9, S21_sliced_mag[start_idx][:,2], color= colors[start_idx], linewidth=3)
    start_idx += 1

plt.plot(S21_sliced_mag[0][:,1]*1E-9, S21_sliced_mag[start_idx][:,2], color= colors[start_idx], linewidth=3, label="$|S\\rangle$")
plt.plot(S21_triplet_mag[:,0]*1E-9, S21_triplet_mag[:,1], color='#F17720', linewidth=3, label="$|T\\rangle$")
plt.grid()
plt.xlabel('Frequency [GHz]')
plt.ylabel('$|S_{21}|$ [dB]')
plt.legend(loc=3, prop={'size': 16})
plt.xlim([6.9,6.914])



plt.figure(figsize=(6,2.5))
start_idx=0
while start_idx < n-1:
    plt.plot(S21_sliced_phase[0][:,1]*1E-9, S21_sliced_phase[start_idx][:,2], color= colors[start_idx], linewidth=3)
    start_idx += 1

plt.plot(S21_sliced_phase[0][:,1]*1E-9, S21_sliced_phase[start_idx][:,2], color= colors[start_idx], linewidth=3, label="$|S\\rangle$")
plt.plot(S21_triplet_phase[:,0]*1E-9, S21_triplet_phase[:,1], color='#F17720', linewidth=3, label="$|T\\rangle$")
plt.grid()
plt.ylim([-100,100])
plt.xlim([6.9,6.914])
plt.xlabel('Frequency [GHz]')
plt.ylabel('$\\angle S_{21}$ [deg.]')
plt.legend(loc=3, prop={'size': 16})


